package ioio.test.ir;

import ioio.lib.api.IrTransmitter;
import ioio.lib.api.exception.ConnectionLostException;
import ioio.lib.util.BaseIOIOLooper;
import ioio.lib.util.IOIOLooper;
import ioio.lib.util.android.IOIOActivity;
import android.os.Bundle;
import android.view.View;
import android.widget.Button;

/**
 * This is the main activity of the HelloIOIO example application.
 * 
 * It displays a toggle button on the screen, which enables control of the
 * on-board LED. This example shows a very simple usage of the IOIO, by using
 * the {@link IOIOActivity} class. For a more advanced use case, see the
 * HelloIOIOPower example.
 */
public class IrTestActivity extends IOIOActivity {
	private static final String TAG = "IOIOIrActivity";
	private Button btnTvPower_;
	private boolean tvPowerClicked_ = false;
	private Button btnDvdPower_;
	private boolean tvSourceClicked_ = false;
	private Button btnTvSource_;
	private boolean dvdPowerClicked_ = false;
	private Button btnDvdOpenCloseTray_;
	private boolean dvdOpenCloseTrayClicked_ = false;
	private Button btnDvbtPower_;
	private boolean dvbtPowerClicked_ = false;
	private Button btnDvbtEpg_;
	private boolean dvbtEpgClicked_ = false;
	
	
	private final int dvdPowerBursts[][] = {
            { 9032, 4516},
            { 552, 1681},
            { 552, 1681},
            { 552, 552},
            { 552, 552},
            { 552, 552},
            { 552, 1681},
            { 552, 552},
            { 552, 1681},
            { 552, 552},
            { 552, 552},
            { 552, 1681},
            { 552, 1681},
            { 552, 1681},
            { 552, 552},
            { 552, 1681},
            { 552, 552},
            { 552, 1681},
            { 552, 552},
            { 552, 552},
            { 552, 1681},
            { 552, 1681},
            { 552, 552},
            { 552, 552},
            { 552, 1681},
            { 552, 552},
            { 552, 1681},
            { 552, 1681},
            { 552, 552},
            { 552, 552},
            { 552, 1681},
            { 552, 1681},
            { 552, 552},
            { 552, 43982},
            { 9032, 4516},
            { 552, 1681},
            { 552, 1681},
            { 552, 1681},
            { 552, 1681},
            { 552, 552},
            { 552, 1681},
            { 552, 552},
            { 552, 1681},
            { 552, 552},
            { 552, 552},
            { 552, 552},
            { 552, 552},
            { 552, 1681},
            { 552, 552},
            { 552, 1681},
            { 552, 552},
            { 552, 552},
            { 552, 552},
            { 552, 1681},
            { 552, 1681},
            { 552, 1681},
            { 552, 1681},
            { 552, 552},
            { 552, 1681},
            { 552, 1681},
            { 552, 1681},
            { 552, 552},
            { 552, 552},
            { 552, 552},
            { 552, 552},
            { 552, 1681},
            { 552, 552},
            { 552, 0}
	};
	private final int dvdOpenCloseTrayBursts[][] = {
			 { 8700, 4100 },
			 { 552, 1681 },
			 { 552, 1681 },
			 { 552, 552 },
			 { 552, 552 },
			 { 552, 552 },
			 { 552, 1681 },
			 { 552, 552 },
			 { 552, 1681 },
			 { 552, 552 },
			 { 552, 300 },
			 { 552, 1681 },
			 { 552, 1681 },
			 { 552, 1681 },
			 { 552, 552 },
			 { 552, 1681 },
			 { 552, 552 },
			 { 552, 1681 },
			 { 552, 552 },
			 { 552, 552 },
			 { 552, 1681 },
			 { 552, 1681 },
			 { 552, 552 },
			 { 552, 552 },
			 { 552, 1681 },
			 { 552, 552 },
			 { 552, 1681 },
			 { 552, 1681 },
			 { 552, 552 },
			 { 552, 552 },
			 { 552, 1681 },
			 { 552, 1681 },
			 { 552, 552 },
			 { 552, 25100 },
			 { 8700, 4200 },
			 { 552, 1681 },
			 { 552, 1681 },
			 { 552, 1681 },
			 { 552, 1681 },
			 { 552, 500 },
			 { 552, 1681 },
			 { 552, 500 },
			 { 552, 1681 },
			 { 552, 500 },
			 { 552, 552 },
			 { 552, 552 },
			 { 552, 552 },
			 { 552, 1681 },
			 { 552, 552 },
			 { 552, 1681 },
			 { 552, 552 },
			 { 552, 500 },
			 { 552, 1681 },
			 { 552, 1681 },
			 { 552, 552 },
			 { 552, 1681 },
			 { 552, 1681 },
			 { 552, 552 },
			 { 552, 1681 },
			 { 552, 1681 },
			 { 552, 500 },
			 { 552, 500 },
			 { 552, 1681 },
			 { 552, 500 },
			 { 552, 552 },
			 { 552, 1681 },
			 { 552, 552 },
			 { 552, 0 }
	};
	private final int tvPowerBursts[][] = {
			{ 9000, 4500 },
			{ 560, 560 },
			{ 560, 560 },
			{ 560, 560 },
			{ 560, 560 },
			{ 560, 560 },
			{ 560, 560 },
			{ 560, 1690 },
			{ 560, 560 },
			{ 560, 1690 },
			{ 560, 1690 },
			{ 560, 1690 },
			{ 560, 1690 },
			{ 560, 1690 },
			{ 560, 1690 },
			{ 560, 560 },
			{ 560, 1690 },
			{ 560, 560 },
			{ 560, 1690 },
			{ 560, 560 },
			{ 560, 560 },
			{ 560, 1690 },
			{ 560, 560 },
			{ 560, 560 },
			{ 560, 560 },
			{ 560, 1690 },
			{ 560, 560 },
			{ 560, 1690 },
			{ 560, 1690 },
			{ 560, 560 },
			{ 560, 1690 },
			{ 560, 1690 },
			{ 560, 1690 },
			{ 560, 42020 },
			{ 9000, 2250 },
			{ 560, 0 }
	};
	private final int tvSourceBursts[][] = {
			 { 9000, 4500 },
			 { 560, 560 },
			 { 560, 560 },
			 { 560, 560 },
			 { 560, 560 },
			 { 560, 560 },
			 { 560, 560 },
			 { 560, 1690 },
			 { 560, 560 },
			 { 560, 1690 },
			 { 560, 1690 },
			 { 560, 1690 },
			 { 560, 1690 },
			 { 560, 1690 },
			 { 560, 1690 },
			 { 560, 560 },
			 { 560, 1690 },
			 { 560, 560 },
			 { 560, 560 },
			 { 560, 1690 },
			 { 560, 560 },
			 { 560, 1690 },
			 { 560, 560 },
			 { 560, 560 },
			 { 560, 560 },
			 { 560, 1690 },
			 { 560, 1690 },
			 { 560, 560 },
			 { 560, 1690 },
			 { 560, 560 },
			 { 560, 1690 },
			 { 560, 1690 },
			 { 560, 1690 },
			 { 560, 41700 },
			 { 9000, 2250 },
			 { 560, 0 }
	};
	private final int dvbtPowerBursts[][] = {
			 { 9100, 4300 },
			 { 800, 400 },
			 { 700, 400 },
			 { 700, 1600 },
			 { 700, 400 },
			 { 700, 400 },
			 { 700, 400 },
			 { 700, 400 },
			 { 800, 400 },
			 { 700, 1500 },
			 { 700, 1600 },
			 { 700, 400 },
			 { 700, 1600 },
			 { 600, 1600 },
			 { 700, 1500 },
			 { 700, 1600 },
			 { 700, 1500 },
			 { 700, 500 },
			 { 600, 1600 },
			 { 700, 400 },
			 { 700, 1600 },
			 { 700, 1500 },
			 { 700, 500 },
			 { 600, 500 },
			 { 700, 400 },
			 { 700, 1600 },
			 { 600, 500 },
			 { 700, 1500 },
			 { 700, 400 },
			 { 700, 500 },
			 { 600, 1600 },
			 { 700, 1600 },
			 { 600, 1600 },
			 { 700, 39800 },
			 { 9100, 2100 },
			 { 700, 0 }
	};
	private final int dvbtEpgBursts[][] = {
			 { 9000, 4500 },
			 { 560, 560 },
			 { 560, 560 },
			 { 560, 1690 },
			 { 560, 560 },
			 { 560, 560 },
			 { 560, 560 },
			 { 560, 560 },
			 { 560, 560 },
			 { 560, 1690 },
			 { 560, 1690 },
			 { 560, 560 },
			 { 560, 1690 },
			 { 560, 1690 },
			 { 560, 1690 },
			 { 560, 1690 },
			 { 560, 1690 },
			 { 560, 1690 },
			 { 560, 1690 },
			 { 560, 560 },
			 { 560, 560 },
			 { 560, 1690 },
			 { 560, 560 },
			 { 560, 1690 },
			 { 560, 560 },
			 { 560, 560 },
			 { 560, 560 },
			 { 560, 1690 },
			 { 560, 1690 },
			 { 560, 560 },
			 { 560, 1690 },
			 { 560, 560 },
			 { 560, 1690 },
			 { 560, 39800 },
			 { 9100, 2200 },
			 { 560, 0 }
	};
	
	/**
	 * Called when the activity is first created. Here we normally initialize
	 * our GUI.
	 */
	@Override
	public void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		setContentView(R.layout.main);
		
		btnTvPower_ = (Button) findViewById(R.id.btnTvPower);
		btnTvPower_.setOnClickListener(new View.OnClickListener() {
			
			@Override
			public void onClick(View v) {
				tvPowerClicked_ = true;
			}
		});
		
		btnTvSource_ = (Button) findViewById(R.id.btnTvSource);
		btnTvSource_.setOnClickListener(new View.OnClickListener() {
			
			@Override
			public void onClick(View v) {
				tvSourceClicked_ = true;
			}
		});
		
		btnDvdPower_ = (Button) findViewById(R.id.btnDvdPower);
		btnDvdPower_.setOnClickListener(new View.OnClickListener() {
			
			@Override
			public void onClick(View v) {
				dvdPowerClicked_ = true;
			}
		});
		
		btnDvdOpenCloseTray_ = (Button) findViewById(R.id.btnDvdOpenCloseTray);
		btnDvdOpenCloseTray_.setOnClickListener(new View.OnClickListener() {
			
			@Override
			public void onClick(View v) {
				dvdOpenCloseTrayClicked_ = true;
			}
		});
		
		btnDvbtPower_ = (Button) findViewById(R.id.btnDvbtPower);
		btnDvbtPower_.setOnClickListener(new View.OnClickListener() {
			
			@Override
			public void onClick(View v) {
				dvbtPowerClicked_ = true;
			}
		});
		
		btnDvbtEpg_ = (Button) findViewById(R.id.btnDvbtEpg);
		btnDvbtEpg_.setOnClickListener(new View.OnClickListener() {
			
			@Override
			public void onClick(View v) {
				dvbtEpgClicked_ = true;
			}
		});
	}

	/**
	 * This is the thread on which all the IOIO activity happens. It will be run
	 * every time the application is resumed and aborted when it is paused. The
	 * method setup() will be called right after a connection with the IOIO has
	 * been established (which might happen several times!). Then, loop() will
	 * be called repetitively until the IOIO gets disconnected.
	 */
	class Looper extends BaseIOIOLooper {
		private IrTransmitter irt_;

		/**
		 * Called every time a connection with IOIO has been established.
		 * Typically used to open pins.
		 * 
		 * @throws ConnectionLostException
		 *             When IOIO connection is lost.
		 * 
		 * @see ioio.lib.util.AbstractIOIOActivity.IOIOThread#setup()
		 */
		@Override
		protected void setup() throws ConnectionLostException {
			irt_ = ioio_.openIrTransmitter(10);
		}

		/**
		 * Called repetitively while the IOIO is connected.
		 * 
		 * @throws ConnectionLostException
		 *             When IOIO connection is lost.
		 * 
		 * @see ioio.lib.util.AbstractIOIOActivity.IOIOThread#loop()
		 */
		@Override
		public void loop() throws ConnectionLostException {
				
			if (tvPowerClicked_) {
				tvPowerClicked_ = false;
				irt_.irTransmit(38000, tvPowerBursts);
			}
			
			if (tvSourceClicked_) {
				tvSourceClicked_ = false;
				irt_.irTransmit(38000, tvSourceBursts);
			}
			
			if (dvdPowerClicked_) {
				dvdPowerClicked_ = false;
				irt_.irTransmit(38000, dvdPowerBursts);
			}
			
			if (dvdOpenCloseTrayClicked_) {
				dvdOpenCloseTrayClicked_ = false;
				irt_.irTransmit(38000, dvdOpenCloseTrayBursts);
			}
			
			if (dvbtPowerClicked_) {
				dvbtPowerClicked_ = false;
				irt_.irTransmit(38000, dvbtPowerBursts);
			}
			
			if (dvbtEpgClicked_) {
				dvbtEpgClicked_ = false;
				irt_.irTransmit(38000, dvbtEpgBursts);
			}
			
			try {
				Thread.sleep(100);
			} catch (InterruptedException e) {
			}
		}			
	}

	/**
	 * A method to create our IOIO thread.
	 * 
	 * @see ioio.lib.util.AbstractIOIOActivity#createIOIOThread()
	 */
	@Override
	protected IOIOLooper createIOIOLooper() {
		return new Looper();
	}
}