#include "delay.h"

#include "GenericTypeDefs.h"
#include "HardwareProfile.h"
#include "logging.h"

static unsigned int iterations_per_10_usecs = 1;

void Delay(unsigned long count) {
    unsigned long iters = count * iterations_per_10_usecs;
    while (iters--);
}

void DelayInit() {
    BYTE ipl_save = SRbits.IPL;
    SRbits.IPL = 7;         //disable interrupts

    TMR1 = 0x0;             // reset timer
    PR1 = 0xFFFF;           // set period
    T1CONbits.TCKPS = 0x0;  // input clock prescale 1:1
    T1CONbits.TCS = 0x0;    // clock source: internal clock (Fosc / 2)
    T1CONbits.TON = 1;      // activate timer

    Delay(100);

    T1CONbits.TON = 0;      // stop timer
    SRbits.IPL = ipl_save;  // enable interrupts

    /* The number of cycles done at 1000us is (GetInstructionClock() / 1000).
     * the approximate actual cycles needed for the delay is TMR1. Calibrate the
     * "usecs to delay loop iterations" using this information. */
    iterations_per_10_usecs = (GetInstructionClock() / 1000) / TMR1;


    // log_printf("Iterations required per 10us: %u", iterations_per_10_usecs);
}
