#include "ir.h"

#include <string.h>
#include <stdint.h>
#include "delay.h"
#include "HardwareProfile.h"
#include "pwm.h"
#include "timer.h"
#include "logging.h"
#include "byte_queue.h"

#define MAX_BURST_PAIRS    50
static WORD buf[IR_BYTES_PER_BURST_PAIR * MAX_BURST_PAIRS];
static int buf_cursor = 0;
static int total_bursts;
static int pwm_num;

#define FreqToPeriod(freq)  (GetInstructionClock() / (freq) - 1)

static void Send(WORD freq)
{
    int i;
    
    // set PWM frequency
    int period = FreqToPeriod(freq);
    int duty_cycle = period / 2;
    SetPwmPeriod(0, period, 0);
    SetPwmDutyCycle(0, duty_cycle, 0);

    log_printf("emitting IR signal at %u Hz using PWM number %d ***", freq, pwm_num);
    for (i = 0; i < total_bursts; i++) {
        PwmEnable(pwm_num);
        Delay(buf[2*i] / 10);
        PwmDisable(pwm_num);
        Delay(buf[2*i+1] / 10);
    }
}

void IrTransmit(WORD freq, BOOL partial, BYTE num_bursts, BYTE *bursts)
{
    log_printf("entering IrTransmit(): freq %u, partial %u, num_bursts %u", freq, partial, num_bursts);
    memcpy(((BYTE*) buf) + buf_cursor, bursts, num_bursts * IR_BYTES_PER_BURST_PAIR);
    total_bursts += num_bursts;
    buf_cursor += num_bursts * IR_BYTES_PER_BURST_PAIR;
    if (!partial) {
        Send(freq);
        total_bursts = 0;
        buf_cursor = 0;
    }
    return;
}

void SetIrPwmModule(BYTE pwm) {
    pwm_num = pwm;
}
